@@ -1,17 +1,15 @@
 # -*- coding: utf-8 -*-
 # Description: smart netdata python.d module
-# Author: l2isbad, vorph1
+# Author: ilyam8, vorph1
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 import os
 import re
-
 from copy import deepcopy
 from time import time
 
-from bases.collection import read_last_line
 from bases.FrameworkServices.SimpleService import SimpleService
-
+from bases.collection import read_last_line
 
 INCREMENTAL = 'incremental'
 ABSOLUTE = 'absolute'
@@ -41,6 +39,7 @@ ATTR171 = '171'
 ATTR172 = '172'
 ATTR173 = '173'
 ATTR174 = '174'
+ATTR177 = '177'
 ATTR180 = '180'
 ATTR183 = '183'
 ATTR190 = '190'
@@ -51,6 +50,10 @@ ATTR198 = '198'
 ATTR199 = '199'
 ATTR202 = '202'
 ATTR206 = '206'
+ATTR233 = '233'
+ATTR241 = '241'
+ATTR242 = '242'
+ATTR249 = '249'
 ATTR_READ_ERR_COR = 'read-total-err-corrected'
 ATTR_READ_ERR_UNC = 'read-total-unc-errors'
 ATTR_WRITE_ERR_COR = 'write-total-err-corrected'
@@ -59,7 +62,6 @@ ATTR_VERIFY_ERR_COR = 'verify-total-err-corrected'
 ATTR_VERIFY_ERR_UNC = 'verify-total-unc-errors'
 ATTR_TEMPERATURE = 'temperature'
 
-
 RE_ATA = re.compile(
     '(\d+);'  # attribute
     '(\d+);'  # normalized value
@@ -114,6 +116,9 @@ ORDER = [
     'current_pending_sector_count',
     'offline_uncorrectable_sector_count',
     'percent_lifetime_used',
+    'media_wearout_indicator',
+    'total_lbas_written',
+    'total_lbas_read',
 ]
 
 CHARTS = {
@@ -154,7 +159,7 @@ CHARTS = {
         'algo': INCREMENTAL,
     },
     'write_total_err_corrected': {
-        'options': [None, 'Write Error Corrected', 'errors', 'errors', 'smartd_log.read_total_err_corrected', 'line'],
+        'options': [None, 'Write Error Corrected', 'errors', 'errors', 'smartd_log.write_total_err_corrected', 'line'],
         'lines': [],
         'attrs': [ATTR_WRITE_ERR_COR],
         'algo': INCREMENTAL,
@@ -265,10 +270,10 @@ CHARTS = {
                     'line'],
         'lines': [],
         'attrs': [ATTR5],
-        'algo': INCREMENTAL,
+        'algo': ABSOLUTE,
     },
     'reserved_block_count': {
-        'options': [None, 'Reserved Block Count', '%', 'wear', 'smartd_log.reserved_block_count', 'line'],
+        'options': [None, 'Reserved Block Count', 'percentage', 'wear', 'smartd_log.reserved_block_count', 'line'],
         'lines': [],
         'attrs': [ATTR170],
         'algo': ABSOLUTE,
@@ -321,11 +326,35 @@ CHARTS = {
 
     },
     'percent_lifetime_used': {
-        'options': [None, 'Percent Lifetime Used', '%', 'wear', 'smartd_log.percent_lifetime_used', 'line'],
+        'options': [None, 'Percent Lifetime Used', 'percentage', 'wear', 'smartd_log.percent_lifetime_used', 'line'],
         'lines': [],
         'attrs': [ATTR202],
         'algo': ABSOLUTE,
-    }
+    },
+    'media_wearout_indicator': {
+        'options': [None, 'Media Wearout Indicator', 'percentage', 'wear', 'smartd_log.media_wearout_indicator', 'line'],
+        'lines': [],
+        'attrs': [ATTR233, ATTR177],
+        'algo': ABSOLUTE,
+    },
+    'nand_writes_1gib': {
+        'options': [None, 'NAND Writes', 'GiB', 'wear', 'smartd_log.nand_writes_1gib', 'line'],
+        'lines': [],
+        'attrs': [ATTR249],
+        'algo': ABSOLUTE,
+    },
+    'total_lbas_written': {
+        'options': [None, 'Total LBAs Written', 'sectors', 'wear', 'smartd_log.total_lbas_written', 'line'],
+        'lines': [],
+        'attrs': [ATTR241],
+        'algo': ABSOLUTE,
+    },
+    'total_lbas_read': {
+        'options': [None, 'Total LBAs Read', 'sectors', 'wear', 'smartd_log.total_lbas_read', 'line'],
+        'lines': [],
+        'attrs': [ATTR242],
+        'algo': ABSOLUTE,
+    },
 }
 
 # NOTE: 'parse_temp' decodes ATA 194 raw value. Not heavily tested. Written by @Ferroin
@@ -440,6 +469,20 @@ class AtaNormalized(BaseAtaSmartAttribute):
         return self.normalized_value
 
 
+class Ata3(BaseAtaSmartAttribute):
+    def value(self):
+        value = int(self.raw_value)
+        # https://github.com/netdata/netdata/issues/5919
+        #
+        # 3;151;38684000679;
+        # 423 (Average 447)
+        # 38684000679 & 0xFFF -> 423
+        # (38684000679 & 0xFFF0000) >> 16 -> 447
+        if value > 1e6:
+            return value & 0xFFF
+        return value
+
+
 class Ata9(BaseAtaSmartAttribute):
     def value(self):
         value = int(self.raw_value)
@@ -453,6 +496,18 @@ class Ata190(BaseAtaSmartAttribute):
         return 100 - int(self.normalized_value)
 
 
+class Ata194(BaseAtaSmartAttribute):
+    # https://github.com/netdata/netdata/issues/3041
+    # https://github.com/netdata/netdata/issues/5919
+    #
+    # The low byte is the current temperature, the third lowest is the maximum, and the fifth lowest is the minimum
+    def value(self):
+        value = int(self.raw_value)
+        if value > 1e6:
+            return value & 0xFF
+        return min(int(self.normalized_value), int(self.raw_value))
+
+
 class BaseSCSISmartAttribute:
     def __init__(self, name, raw_value):
         self.name = name
@@ -470,16 +525,21 @@ class SCSIRaw(BaseSCSISmartAttribute):
 def ata_attribute_factory(value):
     name = value[0]
 
-    if name == ATTR9:
+    if name == ATTR3:
+        return Ata3(*value)
+    elif name == ATTR9:
         return Ata9(*value)
     elif name == ATTR190:
         return Ata190(*value)
+    elif name == ATTR194:
+        return Ata194(*value)
     elif name in [
         ATTR1,
         ATTR7,
-        ATTR194,
+        ATTR177,
         ATTR202,
         ATTR206,
+        ATTR233,
     ]:
         return AtaNormalized(*value)
 
@@ -504,7 +564,9 @@ def handle_error(*errors):
                 return method(*args)
             except errors:
                 return None
+
         return on_call
+
     return on_method
 
 
@@ -529,6 +591,7 @@ class DiskLogFile:
 
 class BaseDisk:
     def __init__(self, name, log_file):
+        self.raw_name = name
         self.name = re.sub(r'_+', '_', name)
         self.log_file = log_file
         self.attrs = list()
@@ -537,8 +600,8 @@ class BaseDisk:
 
     def __eq__(self, other):
         if isinstance(other, BaseDisk):
-            return self.name == other.name
-        return self.name == other
+            return self.raw_name == other.raw_name
+        return self.raw_name == other
 
     def __ne__(self, other):
         return not self == other
@@ -580,13 +643,12 @@ class Service(SimpleService):
         SimpleService.__init__(self, configuration=configuration, name=name)
         self.order = ORDER
         self.definitions = deepcopy(CHARTS)
-
         self.log_path = configuration.get('log_path', DEF_PATH)
         self.age = configuration.get('age', DEF_AGE)
         self.exclude = configuration.get('exclude_disks', str()).split()
-
         self.disks = list()
         self.runs = 0
+        self.do_force_rescan = False
 
     def check(self):
         return self.scan() > 0
@@ -594,9 +656,10 @@ class Service(SimpleService):
     def get_data(self):
         self.runs += 1
 
-        if self.runs % DEF_RESCAN_INTERVAL == 0:
+        if self.do_force_rescan or self.runs % DEF_RESCAN_INTERVAL == 0:
             self.cleanup()
             self.scan()
+            self.do_force_rescan = False
 
         data = dict()
 
@@ -611,10 +674,12 @@ class Service(SimpleService):
 
             if changed is None:
                 disk.alive = False
+                self.do_force_rescan = True
                 continue
 
             if changed and disk.populate_attrs() is None:
                 disk.alive = False
+                self.do_force_rescan = True
                 continue
 
             data.update(disk.data())
@@ -625,12 +690,12 @@ class Service(SimpleService):
         current_time = time()
         for disk in self.disks[:]:
             if any(
-                [
-                    not disk.alive,
-                    not disk.log_file.is_active(current_time, self.age),
-                ]
+                    [
+                        not disk.alive,
+                        not disk.log_file.is_active(current_time, self.age),
+                    ]
             ):
-                self.disks.remove(disk.name)
+                self.disks.remove(disk.raw_name)
                 self.remove_disk_from_charts(disk)
 
     def scan(self):
@@ -645,18 +710,20 @@ class Service(SimpleService):
 
         return len(self.disks)
 
-    def create_disk_from_file(self, full_name,  current_time):
+    def create_disk_from_file(self, full_name, current_time):
+        if not full_name.endswith(CSV):
+            self.debug('skipping {0}: not a csv file'.format(full_name))
+            return None
+
         name = os.path.basename(full_name).split('.')[-3]
         path = os.path.join(self.log_path, full_name)
 
         if name in self.disks:
+            self.debug('skipping {0}: already in disks'.format(full_name))
             return None
 
         if [p for p in self.exclude if p in name]:
-            return None
-
-        if not full_name.endswith(CSV):
-            self.debug('skipping {0}: not a csv file'.format(full_name))
+            self.debug('skipping {0}: filtered by `exclude` option'.format(full_name))
             return None
 
         if not os.access(path, os.R_OK):
@@ -720,5 +787,4 @@ class Service(SimpleService):
             if not chart_id or chart_id not in self.charts:
                 continue
 
-            # TODO: can't delete dimension
-            self.charts[chart_id].hide_dimension('{0}_{1}'.format(disk.name, attr.name))
+            self.charts[chart_id].del_dimension('{0}_{1}'.format(disk.name, attr.name))
